@@ -3,8 +3,11 @@ from __future__ import (unicode_literals, absolute_import,
                         print_function, division)
 
 from pythonpy.pyeval import pyeval
-import sys
+import pydoc
 
 def main():
-    out = pyeval(argv=sys.argv[1:])
-    print(out, end='')
\ No newline at end of file
+    out, pager = pyeval()
+    if pager:
+        pydoc.pager(out)
+    else:
+        print(out, end='')
